#ifndef _ADCREG_H
#define _ADCREG_H

#include "common.h"

/** 
  * @brief Analog to Digital Converter  
  */
/********************  Bits definition for ADC_ISR register  ******************/
typedef struct {
__IO ONEBIT ADRDY     :1;    /*!< ADC Ready */
__IO ONEBIT EOSMP     :1;    /*!< End of sampling flag */
__IO ONEBIT EOC       :1;    /*!< End of Conversion */
__IO ONEBIT EOSEQ     :1;    /*!< End of Sequence flag */
__IO ONEBIT OVR       :1;    /*!< Overrun flag */
     uint32_t UNUSED1   :2;
__IO ONEBIT AWD       :1;    /*!< Analog watchdog flag */
     uint32_t UNUSED2   :24;
} ADC_ISR;
/********************  Bits definition for ADC_IER register  ******************/
typedef struct {
__IO ONEBIT ADRDYIE   :1;    /*!< ADC Ready interrupt enable */
__IO ONEBIT EOSMPIE   :1;    /*!< End of sampling interrupt enable */
__IO ONEBIT EOCIE     :1;    /*!< End of Conversion interrupt enable */
__IO ONEBIT EOSEQIE   :1;    /*!< End of Sequence of conversion interrupt enable */
__IO ONEBIT OVRIE     :1;    /*!< Overrun interrupt enable */  
     uint32_t UNUSED1   :2;
__IO ONEBIT AWDIE     :1;    /*!< Analog Watchdog interrupt enable */
     uint32_t UNUSED2   :24;
} ADC_IER;
/********************  Bits definition for ADC_CR register  *******************/
typedef struct {
__IO ONEBIT ADEN           :1;    /*!< ADC enable control */
__IO ONEBIT ADDIS          :1;    /*!< ADC disable command */
__IO ONEBIT ADSTART        :1;    /*!< ADC start of conversion */
     uint32_t UNUSED1      :1;
__IO ONEBIT ADSTP          :1;    /*!< ADC stop of conversion command */
     uint32_t UNUSED2        :26;
__IO ONEBIT ADCAL          :1;    /*!< ADC calibration */
} ADC_CR;
/*******************  Bits definition for ADC_CFGR1 register  *****************/
typedef struct {
__IO ONEBIT  DMAEN    :1;         /*!< Direct memory access enable */
__IO ONEBIT  DMACFG   :1;         /*!< Direct memory access configuration */
__IO ONEBIT  SCANDIR  :1;         /*!< Sequence scan direction */
__IO uint32_t  RES      :2;         /*!< RES[1:0] bits (Resolution) */
__IO ONEBIT  ALIGN    :1;         /*!< Data Alignment */
__IO uint32_t  EXTSEL   :3;         /*!< EXTSEL[2:0] bits (External Event Select for regular group) */
     uint32_t  UNUSED1:1;
__IO uint32_t  EXTEN    :2;         /*!< EXTEN[1:0] bits (External Trigger Conversion mode for regular channels) */
__IO ONEBIT  OVRMOD   :1;         /*!< Overrun mode */
__IO ONEBIT  CONT     :1;         /*!< Continuous Conversion */
__IO ONEBIT  WAIT     :1;         /*!< ADC wait conversion mode */
__IO ONEBIT  AUTOFF   :1;         /*!< ADC auto power off */

__IO ONEBIT  DISCEN   :1;         /*!< Discontinuous mode on regular channels */
     uint32_t  UNUSED2  :5;
__IO ONEBIT  AWDSGL   :1;         /*!< Enable the watchdog on a single channel or on all channels  */
__IO ONEBIT  AWDEN    :1;         /*!< Analog watchdog enable on regular channels */
     uint32_t  UNUSED3  :2;
__IO uint32_t  AWDCH    :5;         /*!< AWDCH[4:0] bits (Analog watchdog channel select bits) */
     uint32_t  UNUSED4:1;
} ADC_CFGR1;
/*******************  Bits definition for ADC_CFGR2 register  *****************/
typedef struct {
  __IO uint32_t UNUSED : 30;
  __IO uint32_t CKMODE : 2;
} ADC_CFGR2;

typedef union {
  __IO uint32_t  VAL;
  __IO ADC_ISR   BIT;
} ADC_ISR_REG;
typedef union {
  __IO uint32_t  VAL;
  __IO ADC_IER   BIT;
} ADC_IER_REG;

typedef union {
  __IO uint32_t  VAL;
  __IO ADC_CR    BIT;
} ADC_CR_REG;
typedef union {
  __IO uint32_t  VAL;
  __IO ADC_CFGR1 BIT;
} ADC_CFGR1_REG;
typedef union {
  __IO uint32_t  VAL;
  __IO ADC_CFGR2 BIT;
} ADC_CFGR2_REG;

typedef struct {
  __IO ADC_ISR_REG    ISR;    /*!< ADC Interrupt and Status register,                          Address offset:0x00 */
  __IO ADC_IER_REG    IER;    /*!< ADC Interrupt Enable register,                              Address offset:0x04 */
  __IO ADC_CR_REG     CR;     /*!< ADC Control register,                                       Address offset:0x08 */
  __IO ADC_CFGR1_REG  CFGR1;  /*!< ADC Configuration register 1,                               Address offset:0x0C */
  __IO ADC_CFGR2_REG  CFGR2;  /*!< ADC Configuration register 2,                               Address offset:0x10 */
  __IO uint32_t SMPR;         /*!< ADC Sampling time register,                                 Address offset:0x14 */
  uint32_t   RESERVED1;       /*!< Reserved,                                                                  0x18 */
  uint32_t   RESERVED2;       /*!< Reserved,                                                                  0x1C */
  __IO uint32_t TR;           /*!< ADC watchdog threshold register,                            Address offset:0x20 */
  uint32_t   RESERVED3;       /*!< Reserved,                                                                  0x24 */
  __IO uint32_t CHSELR;       /*!< ADC channel selection register,                             Address offset:0x28 */
  uint32_t   RESERVED4[5];    /*!< Reserved,                                                                  0x2C */
   __IO uint32_t DR;          /*!< ADC data register,                                          Address offset:0x40 */
} ADC_Type;

#endif // _ADCREG_H
