#ifndef _DMAREG_H
#define _DMAREG_H

#include "common.h"

/** 
  * @brief DMA Controller
  */
/*******************  Bit definition for DMA_CCR register  ********************/
typedef enum {
  XFER_BYTE = 0, XFER_HALF, XFER_WORD
} DMA_XFER;
typedef struct {
__IO ONEBIT  EN           :1;    /*!< Channel enable                      */
__IO ONEBIT  TCIE         :1;    /*!< Transfer complete interrupt enable  */
__IO ONEBIT  HTIE         :1;    /*!< Half Transfer interrupt enable      */
__IO ONEBIT  TEIE         :1;    /*!< Transfer error interrupt enable     */
__IO ONEBIT  DIR          :1;    /*!< Data transfer direction             */
__IO ONEBIT  CIRC         :1;    /*!< Circular mode                       */
__IO ONEBIT  PINC         :1;    /*!< Peripheral increment mode           */
__IO ONEBIT  MINC         :1;    /*!< Memory increment mode               */

__IO DMA_XFER  PSIZE        :2;    /*!< PSIZE[1:0] bits (Peripheral size)   */
__IO DMA_XFER  MSIZE        :2;    /*!< MSIZE[1:0] bits (Memory size)       */
__IO uint32_t  PL           :2;    /*!< PL[1:0] bits(Channel Priority level)*/
__IO ONEBIT  MEM2MEM      :1;    /*!< Memory to memory mode               */
     uint32_t  UNUSED1      :17;
} DMA_CCR;
typedef union {
  __IO uint32_t VAL;
  __IO DMA_CCR  BIT;
} DMA_CCR_REG;

typedef struct {
  __IO DMA_CCR_REG CCR;          /*!< DMA channel x configuration register                                           */
  __IO uint32_t    CNDTR;        /*!< DMA channel x number of data register                                          */
  __IO uint32_t    CPAR;         /*!< DMA channel x peripheral address register                                      */
  __IO uint32_t    CMAR;         /*!< DMA channel x memory address register                                          */
} DMA_Channel_Type;
/*******************  Bit definition for DMA_ISR register  ********************/
typedef struct {
__IO ONEBIT  GIF1  :1;    /*!< Channel 1 Global interrupt flag    */
__IO ONEBIT  TCIF1 :1;    /*!< Channel 1 Transfer Complete flag   */
__IO ONEBIT  HTIF1 :1;    /*!< Channel 1 Half Transfer flag       */
__IO ONEBIT  TEIF1 :1;    /*!< Channel 1 Transfer Error flag      */
__IO ONEBIT  GIF2  :1;    /*!< Channel 2 Global interrupt flag    */
__IO ONEBIT  TCIF2 :1;    /*!< Channel 2 Transfer Complete flag   */
__IO ONEBIT  HTIF2 :1;    /*!< Channel 2 Half Transfer flag       */
__IO ONEBIT  TEIF2 :1;    /*!< Channel 2 Transfer Error flag      */

__IO ONEBIT  GIF3  :1;    /*!< Channel 3 Global interrupt flag    */
__IO ONEBIT  TCIF3 :1;    /*!< Channel 3 Transfer Complete flag   */
__IO ONEBIT  HTIF3 :1;    /*!< Channel 3 Half Transfer flag       */
__IO ONEBIT  TEIF3 :1;    /*!< Channel 3 Transfer Error flag      */
__IO ONEBIT  GIF4  :1;    /*!< Channel 4 Global interrupt flag    */
__IO ONEBIT  TCIF4 :1;    /*!< Channel 4 Transfer Complete flag   */
__IO ONEBIT  HTIF4 :1;    /*!< Channel 4 Half Transfer flag       */
__IO ONEBIT  TEIF4 :1;    /*!< Channel 4 Transfer Error flag      */

__IO ONEBIT  GIF5  :1;    /*!< Channel 5 Global interrupt flag    */
__IO ONEBIT  TCIF5 :1;    /*!< Channel 5 Transfer Complete flag   */
__IO ONEBIT  HTIF5 :1;    /*!< Channel 5 Half Transfer flag       */
__IO ONEBIT  TEIF5 :1;    /*!< Channel 5 Transfer Error flag      */
__IO ONEBIT  GIF6  :1;    /*!< Channel 6 Global interrupt flag (only for STM32F072 devices) */
__IO ONEBIT  TCIF6 :1;    /*!< Channel 6 Transfer Complete flag (only for STM32F072 devices) */
__IO ONEBIT  HTIF6 :1;    /*!< Channel 6 Half Transfer flag (only for STM32F072 devices) */
__IO ONEBIT  TEIF6 :1;    /*!< Channel 6 Transfer Error flag (only for STM32F072 devices) */

__IO ONEBIT  GIF7  :1;    /*!< Channel 7 Global interrupt flag (only for STM32F072 devices) */
__IO ONEBIT  TCIF7 :1;    /*!< Channel 7 Transfer Complete flag (only for STM32F072 devices) */
__IO ONEBIT  HTIF7 :1;    /*!< Channel 7 Half Transfer flag (only for STM32F072 devices) */
__IO ONEBIT  TEIF7 :1;    /*!< Channel 7 Transfer Error flag (only for STM32F072 devices) */
     uint32_t UNUSED1 :4;
} DMA_ISR;
typedef union {
  __IO uint32_t VAL;
  __IO DMA_ISR  BIT;
} DMA_ISR_REG;

typedef struct {
  __IO DMA_ISR_REG ISR;          /*!< DMA interrupt status register,       Address offset: 0x00 */
  __IO uint32_t    IFCR;         /*!< DMA interrupt flag clear register,   Address offset: 0x04 */
} DMA_Type;

#endif // _DMAREG_H
