#ifndef FILTER_H
#define FILTER_H

#define ISHIFT 12

#ifdef __cplusplus
#include "processdata.h"
#include "fifo.h"
#include "gpio.h"
#include "usart1.h"
#include "print.h"

#define SYNC_HI   (17)  //!< 2.33 sec délka startovacího pulsu
#define SYNC_LO   (22)  //!< 2.99 sec délka synchronizační mezery
#define TBUFLEN   (64)

/**
\enum stat
Stavy konečného automatu vyhodnocení telegramu
*/
enum stat {
  WAIT_FOR_BEGIN = 0,   //!< čekání na začátek telegramu
  SYNC_PULSE,           //!< startovací puls probíhá
  SYNC_SPACE,           //!< synchronizační mezera probíhá
  CORELATE,             //!< datový puls probíhá
};

class Filter : public ProcessData {
  public:
    Filter (int n);
    void Telegram   (void);
    void setTriger  (int val);
    void setOutput  (GpioClass * output);
    void setCommand (const char* command);
    /// Přetížená funkce pro vlastní výpočet algoritmu.
    void Compute   (const uint16_t data);
  protected:
    int  Decode    (int num, char *str);
    void HumanRead (const char* src, char* dst);
    int  Action    (char* tlg, const char* command);
  private:
    volatile int index;
    int          q1,q2;
    int          coeff;       //!< koeficient
    char const*  cmd;

    char      buf1[TBUFLEN];
    char      buf2[TBUFLEN];
    int       suma;
    uint32_t  bits;
    int       counter;            //!< čítač period
    int       triger;             //!< rozhodovací úroveň (napevno)
    stat      status;             //!< stav konečného automatu detekce
    
    
    Usart1Class  serial;
    Print        print;
    Fifo<int>    fifo;
    GpioClass    ledg;
    GpioClass    ledb;
    GpioClass *  relay;
};
#endif // __cplusplus
#endif // FILTER_H
