#ifndef _FLASH_H

#include <stdint.h>
#ifndef __IO
#define __IO volatile
#endif // __IO
/** 
  * @brief FLASH Registers
  */
typedef struct {
  __IO uint32_t ACR;          /*!<FLASH access control register,                 Address offset: 0x00 */
  __IO uint32_t KEYR;         /*!<FLASH key register,                            Address offset: 0x04 */
  __IO uint32_t OPTKEYR;      /*!<FLASH OPT key register,                        Address offset: 0x08 */
  __IO uint32_t SR;           /*!<FLASH status register,                         Address offset: 0x0C */
  __IO uint32_t CR;           /*!<FLASH control register,                        Address offset: 0x10 */
  __IO uint32_t AR;           /*!<FLASH address register,                        Address offset: 0x14 */
  __IO uint32_t RESERVED;     /*!< Reserved,                                                     0x18 */
  __IO uint32_t OBR;          /*!<FLASH option bytes register,                   Address offset: 0x1C */
  __IO uint32_t WRPR;         /*!<FLASH option bytes register,                   Address offset: 0x20 */
} FLASH_Type;

#define  FLASH_ACR_LATENCY  ((uint32_t)0x00000001)        /*!< LATENCY bit (Latency)  */
#define  FLASH_ACR_PRFTBE   ((uint32_t)0x00000010)        /*!< Prefetch Buffer Enable */
#define  FLASH_ACR_PRFTBS   ((uint32_t)0x00000020)        /*!< Prefetch Buffer Status */

#define _FLASH_H
#endif // _FLASH_H