#ifndef _RCC_H
#define _RCC_H

#include "common.h"

#if !defined  (HSE_VALUE)     
#define HSE_VALUE        ((uint32_t)8000000)           /*!< Value of the External oscillator in Hz */
#endif /* HSE_VALUE */
#if !defined  (HSI_VALUE) 
#define HSI_VALUE        ((uint32_t)8000000)           /*!< Value of the Internal High Speed oscillator in Hz. */
#endif /* HSI_VALUE */

#define HSE_STARTUP_TIMEOUT   ((uint16_t)0x5000)       /*!< Time out for HSE start up */

/********************  Bit definition for RCC_CR register  ********************/
typedef struct {
__IO ONEBIT   HSION    :1;        /*!< Internal High Speed clock enable */
__IO ONEBIT   HSIRDY   :1;        /*!< Internal High Speed clock ready flag */
     uint32_t UNUSED1  :1;
__IO uint32_t HSITRIM  :5;        /*!< Internal High Speed clock trimming */

__IO uint32_t HSICAL   :8;        /*!< Internal High Speed clock Calibration */

__IO ONEBIT   HSEON    :1;        /*!< External High Speed clock enable */
__IO ONEBIT   HSERDY   :1;        /*!< External High Speed clock ready flag */
__IO ONEBIT   HSEBYP   :1;        /*!< External High Speed clock Bypass */
__IO ONEBIT   CSSON    :1;        /*!< Clock Security System enable */
     uint32_t UNUSED2  :4;

__IO ONEBIT   PLLON    :1;        /*!< PLL enable */
__IO ONEBIT   PLLRDY   :1;        /*!< PLL clock ready flag */
     uint32_t UNUSED3  :6;
} RCC_CR;

/*******************  Bit definition for RCC_CFGR register  *******************/
typedef enum {
  USEHSI = 0, USEHSE, USEPLL
} SW_EN;
typedef enum {
  RCC_CFGR_PLLMUL2 = 0,
  RCC_CFGR_PLLMUL3, 
  RCC_CFGR_PLLMUL4, 
  RCC_CFGR_PLLMUL5, 
  RCC_CFGR_PLLMUL6, 
  RCC_CFGR_PLLMUL7, 
  RCC_CFGR_PLLMUL8, 
  RCC_CFGR_PLLMUL9, 
  RCC_CFGR_PLLMUL10,
  RCC_CFGR_PLLMUL11,
  RCC_CFGR_PLLMUL12,
  RCC_CFGR_PLLMUL13,
  RCC_CFGR_PLLMUL14,
  RCC_CFGR_PLLMUL15,
  RCC_CFGR_PLLMUL16,
} PLLML_EN;
typedef struct {
__IO SW_EN    SW         :2;       /*!< SW[1:0] bits (System clock Switch) */
__IO SW_EN    SWS        :2;       /*!< SWS[1:0] bits (System Clock Switch Status) */
__IO uint32_t HPRE       :4;       /*!< HPRE[3:0] bits (AHB prescaler) */
__IO uint32_t PPRE       :3;       /*!< PRE[2:0] bits (APB prescaler) */
     uint32_t UNUSED1    :3;
__IO ONEBIT   ADCPRE     :1;       /*!< ADC prescaler: Obsolete.  */
     uint32_t UNUSED2    :1;

__IO ONEBIT   PLLSRC     :1;       /*!< PLL entry clock source */
__IO ONEBIT   PLLXTPRE   :1;       /*!< HSE divider for PLL entry */
__IO PLLML_EN PLLMUL     :4;       /*!< PLLMUL[3:0] bits (PLL multiplication factor) */
     uint32_t UNUSED3    :2;
     
__IO uint32_t MCO        :3;       /*!< MCO[2:0] bits (Microcontroller Clock Output) */
     uint32_t UNUSED4    :1;
__IO uint32_t MCO_PRE    :3;       /*!< MCO prescaler */
__IO ONEBIT   PLLNODIV   :1;       /*!< PLL is not divided to MCO (this bit are not available in the STM32F051 devices) */
} RCC_CFGR;
/*******************  Bit definition for RCC_CIR register  ********************/
typedef struct {
__IO ONEBIT   LSIRDYF    :1;  /*!< LSI Ready Interrupt flag */
__IO ONEBIT   LSERDYF    :1;  /*!< LSE Ready Interrupt flag */
__IO ONEBIT   HSIRDYF    :1;  /*!< HSI Ready Interrupt flag */
__IO ONEBIT   HSERDYF    :1;  /*!< HSE Ready Interrupt flag */
__IO ONEBIT   PLLRDYF    :1;  /*!< PLL Ready Interrupt flag */
__IO ONEBIT   HSI14RDYF  :1;  /*!< HSI14 Ready Interrupt flag */
__IO ONEBIT   HSI48RDYIF :1;  /*!< HSI48 Ready Interrupt flag */
__IO ONEBIT   CSSF       :1;  /*!< Clock Security System Interrupt flag */
  
__IO ONEBIT   LSIRDYIE   :1;  /*!< LSI Ready Interrupt Enable */
__IO ONEBIT   LSERDYIE   :1;  /*!< LSE Ready Interrupt Enable */
__IO ONEBIT   HSIRDYIE   :1;  /*!< HSI Ready Interrupt Enable */
__IO ONEBIT   HSERDYIE   :1;  /*!< HSE Ready Interrupt Enable */
__IO ONEBIT   PLLRDYIE   :1;  /*!< PLL Ready Interrupt Enable */
__IO ONEBIT   HSI14RDYIE :1;  /*!< HSI14 Ready Interrupt Enable */
__IO ONEBIT   HSI48RDYIE :1;  /*!< HSI48 Ready Interrupt Enable */
     uint32_t UNUSED1    :1;

__IO ONEBIT   LSIRDYC    :1;  /*!< LSI Ready Interrupt Clear */
__IO ONEBIT   LSERDYC    :1;  /*!< LSE Ready Interrupt Clear */
__IO ONEBIT   HSIRDYC    :1;  /*!< HSI Ready Interrupt Clear */
__IO ONEBIT   HSERDYC    :1;  /*!< HSE Ready Interrupt Clear */
__IO ONEBIT   PLLRDYC    :1;  /*!< PLL Ready Interrupt Clear */
__IO ONEBIT   HSI14RDYC  :1;  /*!< HSI14 Ready Interrupt Clear */
__IO ONEBIT   HSI48RDYC  :1;  /*!< HSI48 Ready Interrupt Clear */
__IO ONEBIT   CSSC       :1;  /*!< Clock Security System Interrupt Clear */

     uint32_t UNUSED2    :8;
} RCC_CIR;
/*****************  Bit definition for RCC_APB2RSTR register  *****************/
typedef struct {
__IO ONEBIT   SYSCFGRST  :1;  /*!< SYSCFG clock reset */
     uint32_t UNUSED1    :8;
__IO ONEBIT   ADCRST     :1;  /*!< ADC clock reset */
     uint32_t UNUSED2    :1;
__IO ONEBIT   TIM1RST    :1;  /*!< TIM1 clock reset */
__IO ONEBIT   SPI1RST    :1;  /*!< SPI1 clock reset */
     uint32_t UNUSED3    :1;
__IO ONEBIT   USART1RST  :1;  /*!< USART1 clock reset */
     uint32_t UNUSED4    :1;
     
__IO ONEBIT   TIM15RST   :1;  /*!< TIM15 clock reset */
__IO ONEBIT   TIM16RST   :1;  /*!< TIM16 clock reset */
__IO ONEBIT   TIM17RST   :1;  /*!< TIM17 clock reset */
     uint32_t UNUSED5    :3;
__IO ONEBIT   DBGMCURST  :1;  /*!< DBGMCU clock reset */
     uint32_t UNUSED6    :9;    
} RCC_APB2RSTR;

/*****************  Bit definition for RCC_APB1RSTR register  *****************/
typedef struct {
__IO ONEBIT   TIM2RST    :1;  /*!< Timer 2 clock reset */
__IO ONEBIT   TIM3RST    :1;  /*!< Timer 3 clock reset */
     uint32_t UNUSED1    :2;
__IO ONEBIT   TIM6RST    :1;  /*!< Timer 6 clock reset */
     uint32_t UNUSED2    :3;
     
__IO ONEBIT   TIM14RST   :1;  /*!< Timer 14 clock reset */
     uint32_t UNUSED3    :2;
__IO ONEBIT   WWDGRST    :1;  /*!< Window Watchdog clock reset */
     uint32_t UNUSED4    :2;
__IO ONEBIT   SPI2RST    :1;  /*!< SPI2 clock reset */
     uint32_t UNUSED5    :2;
__IO ONEBIT   USART2RST  :1;  /*!< USART 2 clock reset */
     uint32_t UNUSED6    :3;
//__IO ONEBIT   USART3RST  :1;  /*!< USART 3 clock reset */
//__IO ONEBIT   USART4RST  :1;  /*!< USART 4 clock reset */
__IO ONEBIT   I2C1RST    :1;  /*!< I2C 1 clock reset */
__IO ONEBIT   I2C2RST    :1;  /*!< I2C 2 clock reset */
     uint32_t UNUSED7    :5;
//__IO ONEBIT   USBRST     :1;  /*!< USB clock reset */
//__IO ONEBIT   CANRST     :1;  /*!< CAN clock reset */
//__IO ONEBIT   CRSRST     :1;  /*!< CRS clock reset */
__IO ONEBIT   PWRRST     :1;  /*!< PWR clock reset */
__IO ONEBIT   DACRST     :1;  /*!< DAC clock reset */
__IO ONEBIT   CECRST     :1;  /*!< CEC clock reset */
     uint32_t UNUSED8    :1;
} RCC_APB1RSTR;
/******************  Bit definition for RCC_AHBENR register  ******************/
typedef struct {
__IO ONEBIT   DMAEN      :1;  /*!< DMA clock enable */
     uint32_t UNUSED1    :1;
__IO ONEBIT   SRAMEN     :1;  /*!< SRAM interface clock enable */
     uint32_t UNUSED2    :1;
__IO ONEBIT   FLITFEN    :1;  /*!< FLITF clock enable */
     uint32_t UNUSED3    :1;
__IO ONEBIT   CRCEN      :1;  /*!< CRC clock enable */
     uint32_t UNUSED4    :10;
__IO ONEBIT   GPIOAEN    :1;  /*!< GPIOA clock enable */
__IO ONEBIT   GPIOBEN    :1;  /*!< GPIOB clock enable */
__IO ONEBIT   GPIOCEN    :1;  /*!< GPIOC clock enable */
__IO ONEBIT   GPIODEN    :1;  /*!< GPIOD clock enable */
__IO ONEBIT   GPIOEEN    :1;  /*!< GPIOE clock enable */
__IO ONEBIT   GPIOFEN    :1;  /*!< GPIOF clock enable */
     uint32_t UNUSED5    :1;
__IO ONEBIT   TSCEN      :1;  /*!< TS controller clock enable */
     uint32_t UNUSED6    :7;
} RCC_AHBENR;
/*****************  Bit definition for RCC_APB2ENR register  ******************/
typedef struct {
__IO ONEBIT   SYSCFGEN     :1;  /*!< SYSCFG and comparator clock enable */
     uint32_t UNUSED1      :8;
__IO ONEBIT   ADCEN        :1;  /*!< ADC1 clock enable */
     uint32_t UNUSED2      :1;
__IO ONEBIT   TIM1EN       :1;  /*!< TIM1 clock enable */
__IO ONEBIT   SPI1EN       :1;  /*!< SPI1 clock enable */
     uint32_t UNUSED3      :1;
__IO ONEBIT   USART1EN     :1;  /*!< USART1 clock enable */
     uint32_t UNUSED4      :1;
     
__IO ONEBIT   TIM15EN      :1;  /*!< TIM15 clock enable */
__IO ONEBIT   TIM16EN      :1;  /*!< TIM16 clock enable */
__IO ONEBIT   TIM17EN      :1;  /*!< TIM17 clock enable */
     uint32_t UNUSED5      :3;
__IO ONEBIT   DBGMCUEN     :1;  /*!< DBGMCU clock enable */
     uint32_t UNUSED6      :9;
} RCC_APB2ENR;
/*****************  Bit definition for RCC_APB1ENR register  ******************/
typedef struct {
__IO ONEBIT   TIM2EN       :1;  /*!< Timer 2 clock enable */
__IO ONEBIT   TIM3EN       :1;  /*!< Timer 3 clock enable */
     uint32_t UNUSED1      :2;
__IO ONEBIT   TIM6EN       :1;  /*!< Timer 6 clock enable */
     uint32_t UNUSED2      :3;
//__IO ONEBIT   TIM7EN       :1;  /*!< Timer 7 clock enable */
     
__IO ONEBIT   TIM14EN      :1;  /*!< Timer 14 clock enable */
     uint32_t UNUSED3      :2;
__IO ONEBIT   WWDGEN       :1;  /*!< Window Watchdog clock enable */
     uint32_t UNUSED4      :2;
__IO ONEBIT   SPI2EN       :1;  /*!< SPI2 clock enable */
     uint32_t UNUSED5      :2;
__IO ONEBIT   USART2EN     :1;  /*!< USART2 clock enable */
     uint32_t UNUSED6      :3;
//__IO ONEBIT   USART3EN     :1;  /*!< USART3 clock enable */
//__IO ONEBIT   USART4EN     :1;  /*!< USART4 clock enable */
__IO ONEBIT   I2C1EN       :1;  /*!< I2C1 clock enable */
__IO ONEBIT   I2C2EN       :1;  /*!< I2C2 clock enable */
     uint32_t UNUSED7      :5;
//__IO ONEBIT   USBEN        :1;  /*!< USB clock enable */
//__IO ONEBIT   CANEN        :1;  /*!< CAN clock enable */
//      __IO ONEBIT   CRSEN        :1;  /*!< CRS clock enable */
__IO ONEBIT   PWREN        :1;  /*!< PWR clock enable */
__IO ONEBIT   DACEN        :1;  /*!< DAC clock enable */
__IO ONEBIT   CECEN        :1;  /*!< CEC clock enable */
     uint32_t UNUSED8      :1;
} RCC_APB1ENR;
/*******************  Bit definition for RCC_BDCR register  *******************/
typedef struct {
__IO ONEBIT   LSEON          :1;  /*!< External Low Speed oscillator enable */
__IO ONEBIT   LSERDY         :1;  /*!< External Low Speed oscillator Ready */
__IO ONEBIT   LSEBYP         :1;  /*!< External Low Speed oscillator Bypass */
__IO uint32_t LSEDRV         :2;  /*!< LSEDRV[1:0] bits (LSE Osc. drive capability) */
     uint32_t UNUSED1        :3;

__IO uint32_t RTCSEL         :2;  /*!< RTCSEL[1:0] bits (RTC clock source selection) */
     uint32_t UNUSED2        :5;
__IO ONEBIT   RTCEN          :1;  /*!< RTC clock enable */

__IO ONEBIT   BDRST          :1;  /*!< Backup domain software reset  */
     uint32_t UNUSED3        :15;
} RCC_BDCR;
/*******************  Bit definition for RCC_CSR register  ********************/  
typedef struct {
__IO ONEBIT   LSION          :1;  /*!< Internal Low Speed oscillator enable */
__IO ONEBIT   LSIRDY         :1;  /*!< Internal Low Speed oscillator Ready */
     uint32_t UNUSED1        :21;
__IO ONEBIT   V18PWRRSTF     :1;  /*!< V1.8 power domain reset flag */
__IO ONEBIT   RMVF           :1;  /*!< Remove reset flag */
__IO ONEBIT   OBLRSTF        :1;  /*!< OBL reset flag */
__IO ONEBIT   PINRSTF        :1;  /*!< PIN reset flag */
__IO ONEBIT   PORRSTF        :1;  /*!< POR/PDR reset flag */
__IO ONEBIT   SFTRSTF        :1;  /*!< Software Reset flag */
__IO ONEBIT   IWDGRSTF       :1;  /*!< Independent Watchdog reset flag */
__IO ONEBIT   WWDGRSTF       :1;  /*!< Window watchdog reset flag */
__IO ONEBIT   LPWRRSTF       :1;  /*!< Low-Power reset flag */

} RCC_CSR;
/*******************  Bit definition for RCC_AHBRSTR register  ****************/
typedef struct {
     uint32_t UNUSED1        :17;
__IO ONEBIT   GPIOARST       :1;  /*!< GPIOA clock reset */
__IO ONEBIT   GPIOBRST       :1;  /*!< GPIOB clock reset */
__IO ONEBIT   GPIOCRST       :1;  /*!< GPIOC clock reset */
__IO ONEBIT   GPIODRST       :1;  /*!< GPIOD clock reset */
__IO ONEBIT   GPIOERST       :1;  /*!< GPIOE clock reset */
__IO ONEBIT   GPIOFRST       :1;  /*!< GPIOF clock reset */
     uint32_t UNUSED2        :1;
__IO ONEBIT   TSCRST         :1;  /*!< TS clock reset */
     uint32_t UNUSED3        :7;
} RCC_AHBRSTR;
/*******************  Bit definition for RCC_CFGR2 register  ******************/
typedef struct {
__IO uint32_t PREDIV         :4;  /*!< PREDIV1[3:0] bits */
     uint32_t UNUSED1        :28;
} RCC_CFGR2;
/*******************  Bit definition for RCC_CFGR3 register  ******************/
typedef struct {
__IO uint32_t USART1SW       :2;  /*!< USART1SW[1:0] bits */
     uint32_t UNUSED1        :2;
__IO ONEBIT   I2C1SW         :1;  /*!< I2C1SW bits */
     uint32_t UNUSED2        :1;
__IO ONEBIT   CECSW          :1;  /*!< CECSW bits */
     uint32_t UNUSED3        :1;
//__IO ONEBIT   USBSW          :1;  /*!< USBSW bits */
__IO ONEBIT   ADCSW          :1;  /*!< ADCSW bits */
     uint32_t UNUSED4        :23;
//__IO ONEBIT   USART2SW       :1;  /*!< USART2SW[1:0] bits */
} RCC_CFGR3;
/*******************  Bit definition for RCC_CR2 register  ********************/
typedef struct {
__IO ONEBIT   HSI14ON        :1;  /*!< Internal High Speed 14MHz clock enable */
__IO ONEBIT   HSI14RDY       :1;  /*!< Internal High Speed 14MHz clock ready flag */
__IO ONEBIT   HSI14DIS       :1;  /*!< Internal High Speed 14MHz clock disable */

__IO uint32_t HSI14TRIM      :5;  /*!< Internal High Speed 14MHz clock trimming */
__IO uint32_t HSI14CAL       :8;  /*!< Internal High Speed 14MHz clock Calibration */
     uint32_t UNUSED1        :16;
//__IO ONEBIT   HSI48ON        :1;  /*!< Internal High Speed 48MHz clock enable */
//__IO ONEBIT   HSI48RDY       :1;  /*!< Internal High Speed 48MHz clock ready flag */
//__IO uint32_t HSI48CAL       :8;  /*!< Internal High Speed 48MHz clock Calibration */
} RCC_CR2;
/************************ Unions for direct value use *************************/
typedef union {
  __IO uint32_t       VAL;
  __IO RCC_CR         BIT;
} RCC_CR_REG;
typedef union {
  __IO uint32_t       VAL;
  __IO RCC_CFGR       BIT;
} RCC_CFGR_REG;
typedef union {
  __IO uint32_t       VAL;
  __IO RCC_CIR        BIT;
} RCC_CIR_REG;
typedef union {
  __IO uint32_t       VAL;
  __IO RCC_APB2RSTR   BIT;
} RCC_APB2RSTR_REG;
typedef union {
  __IO uint32_t       VAL;
  __IO RCC_APB1RSTR   BIT;
} RCC_APB1RSTR_REG;
typedef union {
  __IO uint32_t       VAL;
  __IO RCC_AHBENR     BIT;
} RCC_AHBENR_REG;
typedef union {
  __IO uint32_t       VAL;
  __IO RCC_APB2ENR    BIT;
} RCC_APB2ENR_REG;
typedef union {
  __IO uint32_t       VAL;
  __IO RCC_APB1ENR    BIT;
} RCC_APB1ENR_REG;
typedef union {
  __IO uint32_t       VAL;
  __IO RCC_BDCR       BIT;
} RCC_BDCR_REG;
typedef union {
  __IO uint32_t       VAL;
  __IO RCC_CSR        BIT;
} RCC_CSR_REG;
typedef union {
  __IO uint32_t       VAL;
  __IO RCC_AHBRSTR    BIT;
} RCC_AHBRSTR_REG;
typedef union {
  __IO uint32_t       VAL;
  __IO RCC_CFGR2      BIT;
} RCC_CFGR2_REG;
typedef union {
  __IO uint32_t       VAL;
  __IO RCC_CFGR3      BIT;
} RCC_CFGR3_REG;
typedef union {
  __IO uint32_t       VAL;
  __IO RCC_CR2        BIT;
} RCC_CR2_REG;
/** 
  * @brief Reset and Clock Control
  */
typedef struct {
  __IO  RCC_CR_REG       CR;         /*!< RCC clock control register,                     Address offset: 0x00 */
  __IO  RCC_CFGR_REG     CFGR;       /*!< RCC clock configuration register,               Address offset: 0x04 */
  __IO  RCC_CIR_REG      CIR;        /*!< RCC clock interrupt register,                   Address offset: 0x08 */
  __IO  RCC_APB2RSTR_REG APB2RSTR;   /*!< RCC APB2 peripheral reset register,             Address offset: 0x0C */
  __IO  RCC_APB1RSTR_REG APB1RSTR;   /*!< RCC APB1 peripheral reset register,             Address offset: 0x10 */
  __IO  RCC_AHBENR_REG   AHBENR;     /*!< RCC AHB peripheral clock register,              Address offset: 0x14 */
  __IO  RCC_APB2ENR_REG  APB2ENR;    /*!< RCC APB2 peripheral clock enable register,      Address offset: 0x18 */
  __IO  RCC_APB1ENR_REG  APB1ENR;    /*!< RCC APB1 peripheral clock enable register,      Address offset: 0x1C */
  __IO  RCC_BDCR_REG     BDCR;       /*!< RCC Backup domain control register,             Address offset: 0x20 */ 
  __IO  RCC_CSR_REG      CSR;        /*!< RCC clock control & status register,            Address offset: 0x24 */
  __IO  RCC_AHBRSTR_REG  AHBRSTR;    /*!< RCC AHB peripheral reset register,              Address offset: 0x28 */
  __IO  RCC_CFGR2_REG    CFGR2;      /*!< RCC clock configuration register 2,             Address offset: 0x2C */
  __IO  RCC_CFGR3_REG    CFGR3;      /*!< RCC clock configuration register 3,             Address offset: 0x30 */
  __IO  RCC_CR2_REG      CR2;        /*!< RCC clock control register 2,                   Address offset: 0x34 */
} RCC_Type;

static RCC_Type * const RCC = (RCC_Type *) 0x40021000;
//#define RCC                 ((RCC_Type *) 0x40021000)

#endif // _RCC_H
