#ifndef TIMER_REG_H
#define TIMER_REG_H

#include "common.h"
/** 
  * @brief TIMER REGISTERS
  */

/*******************  Bit definition for TIM_CR1 register  ********************/
typedef struct {
__IO ONEBIT  CEN          :1;  /*!<Counter enable */
__IO ONEBIT  UDIS         :1;  /*!<Update disable */
__IO ONEBIT  URS          :1;  /*!<Update request source */
__IO ONEBIT  OPM          :1;  /*!<One pulse mode */
__IO ONEBIT  DIR          :1;  /*!<Direction */
__IO uint16_t  CMS          :2;  /*!<CMS[1:0] bits (Center-aligned mode selection) */
__IO ONEBIT  ARPE         :1;  /*!<Auto-reload preload enable */
__IO uint16_t  CKD          :2;  /*!<CKD[1:0] bits (clock division) */
} PACKSTRUCT TIM_CR1;
/*******************  Bit definition for TIM_CR2 register  ********************/
typedef struct {
__IO ONEBIT  CCPC         :1;  /*!<Capture/Compare Preloaded Control */
     uint16_t UNUSED1     :1;
__IO ONEBIT  CCUS         :1;  /*!<Capture/Compare Control Update Selection */
__IO ONEBIT  CCDS         :1;  /*!<Capture/Compare DMA Selection */
__IO uint16_t  MMS          :3;  /*!<MMS[2:0] bits (Master Mode Selection) */
__IO ONEBIT  TI1S         :1;  /*!<TI1 Selection */

__IO ONEBIT  OIS1         :1;  /*!<Output Idle state 1 (OC1 output) */
__IO ONEBIT  OIS1N        :1;  /*!<Output Idle state 1 (OC1N output) */
__IO ONEBIT  OIS2         :1;  /*!<Output Idle state 2 (OC2 output) */
__IO ONEBIT  OIS2N        :1;  /*!<Output Idle state 2 (OC2N output) */
__IO ONEBIT  OIS3         :1;  /*!<Output Idle state 3 (OC3 output) */
__IO ONEBIT  OIS3N        :1;  /*!<Output Idle state 3 (OC3N output) */
__IO ONEBIT  OIS4         :1;  /*!<Output Idle state 4 (OC4 output) */
} PACKSTRUCT TIM_CR2;

/*******************  Bit definition for TIM_SMCR register  *******************/
typedef struct {
__IO uint16_t  SMS          :3;  /*!<SMS[2:0] bits (Slave mode selection) */
__IO ONEBIT  OCCS         :1;  /*!< OCREF clear selection */
__IO uint16_t  TS           :3;  /*!<TS[2:0] bits (Trigger selection) */
__IO ONEBIT  MSM          :1;  /*!<Master/slave mode */

__IO uint16_t  ETF          :4;  /*!<ETF[3:0] bits (External trigger filter) */
__IO uint16_t  ETPS         :2;  /*!<ETPS[1:0] bits (External trigger prescaler) */
__IO ONEBIT  ECE          :1;  /*!<External clock enable */
__IO ONEBIT  ETP          :1;  /*!<External trigger polarity */
} PACKSTRUCT TIM_SMCR;
/*******************  Bit definition for TIM_DIER register  *******************/
typedef struct {
__IO ONEBIT  UIE          :1;  /*!<Update interrupt enable */
__IO ONEBIT  CC1IE        :1;  /*!<Capture/Compare 1 interrupt enable */
__IO ONEBIT  CC2IE        :1;  /*!<Capture/Compare 2 interrupt enable */
__IO ONEBIT  CC3IE        :1;  /*!<Capture/Compare 3 interrupt enable */
__IO ONEBIT  CC4IE        :1;  /*!<Capture/Compare 4 interrupt enable */
__IO ONEBIT  COMIE        :1;  /*!<COM interrupt enable */
__IO ONEBIT  TIE          :1;  /*!<Trigger interrupt enable */
__IO ONEBIT  BIE          :1;  /*!<Break interrupt enable */
__IO ONEBIT  UDE          :1;  /*!<Update DMA request enable */
__IO ONEBIT  CC1DE        :1;  /*!<Capture/Compare 1 DMA request enable */
__IO ONEBIT  CC2DE        :1;  /*!<Capture/Compare 2 DMA request enable */
__IO ONEBIT  CC3DE        :1;  /*!<Capture/Compare 3 DMA request enable */
__IO ONEBIT  CC4DE        :1;  /*!<Capture/Compare 4 DMA request enable */
__IO ONEBIT  COMDE        :1;  /*!<COM DMA request enable */
__IO ONEBIT  TDE          :1;  /*!<Trigger DMA request enable */
} PACKSTRUCT TIM_DIER;
/********************  Bit definition for TIM_SR register  ********************/
typedef struct {
__IO ONEBIT  UIF          :1;  /*!<Update interrupt Flag */
__IO ONEBIT  CC1IF        :1;  /*!<Capture/Compare 1 interrupt Flag */
__IO ONEBIT  CC2IF        :1;  /*!<Capture/Compare 2 interrupt Flag */
__IO ONEBIT  CC3IF        :1;  /*!<Capture/Compare 3 interrupt Flag */
__IO ONEBIT  CC4IF        :1;  /*!<Capture/Compare 4 interrupt Flag */
__IO ONEBIT  COMIF        :1;  /*!<COM interrupt Flag */
__IO ONEBIT  TIF          :1;  /*!<Trigger interrupt Flag */
__IO ONEBIT  BIF          :1;  /*!<Break interrupt Flag */

     uint16_t UNUSED1     :1;
__IO ONEBIT  CC1OF        :1;  /*!<Capture/Compare 1 Overcapture Flag */
__IO ONEBIT  CC2OF        :1;  /*!<Capture/Compare 2 Overcapture Flag */
__IO ONEBIT  CC3OF        :1;  /*!<Capture/Compare 3 Overcapture Flag */
__IO ONEBIT  CC4OF        :1;  /*!<Capture/Compare 4 Overcapture Flag */
} PACKSTRUCT TIM_SR;
/*******************  Bit definition for TIM_EGR register  ********************/
typedef struct {
__IO ONEBIT  UG           :1;  /*!<Update Generation */
__IO ONEBIT  CC1G         :1;  /*!<Capture/Compare 1 Generation */
__IO ONEBIT  CC2G         :1;  /*!<Capture/Compare 2 Generation */
__IO ONEBIT  CC3G         :1;  /*!<Capture/Compare 3 Generation */
__IO ONEBIT  CC4G         :1;  /*!<Capture/Compare 4 Generation */
__IO ONEBIT  COMG         :1;  /*!<Capture/Compare Control Update Generation */
__IO ONEBIT  TG           :1;  /*!<Trigger Generation */
__IO ONEBIT  BG           :1;  /*!<Break Generation */
} PACKSTRUCT TIM_EGR;
/******************  Bit definition for TIM_CCMR1 register  *******************/
typedef struct {
__IO uint16_t  CC1S         :2;  /*!<CC1S[1:0] bits (Capture/Compare 1 Selection) */
__IO ONEBIT  OC1FE        :1;  /*!<Output Compare 1 Fast enable */
__IO ONEBIT  OC1PE        :1;  /*!<Output Compare 1 Preload enable */
__IO uint16_t  OC1M         :3;  /*!<OC1M[2:0] bits (Output Compare 1 Mode) */
__IO ONEBIT  OC1CE        :1;  /*!<Output Compare 1Clear Enable */

__IO uint16_t  CC2S         :2;  /*!<CC2S[1:0] bits (Capture/Compare 2 Selection) */
__IO ONEBIT  OC2FE        :1;  /*!<Output Compare 2 Fast enable */
__IO ONEBIT  OC2PE        :1;  /*!<Output Compare 2 Preload enable */
__IO uint16_t  OC2M         :3;  /*!<OC2M[2:0] bits (Output Compare 2 Mode) */
__IO ONEBIT  OC2CE        :1;  /*!<Output Compare 2 Clear Enable */
} PACKSTRUCT TIM_CCMR1;

/******************  Bit definition for TIM_CCMR2 register  *******************/
typedef struct {
__IO uint16_t  CC3S         :2;  /*!<CC3S[1:0] bits (Capture/Compare 3 Selection) */
__IO ONEBIT  OC3FE        :1;  /*!<Output Compare 3 Fast enable */
__IO ONEBIT  OC3PE        :1;  /*!<Output Compare 3 Preload enable */
__IO uint16_t  OC3M         :3;  /*!<OC3M[2:0] bits (Output Compare 3 Mode) */
__IO ONEBIT  OC3CE        :1;  /*!<Output Compare 3 Clear Enable */

__IO uint16_t  CC4S         :2;  /*!<CC4S[1:0] bits (Capture/Compare 4 Selection) */
__IO ONEBIT  OC4FE        :1;  /*!<Output Compare 4 Fast enable */
__IO ONEBIT  OC4PE        :1;  /*!<Output Compare 4 Preload enable */
__IO uint16_t  OC4M         :3;  /*!<OC4M[2:0] bits (Output Compare 4 Mode) */
__IO ONEBIT  OC4CE        :1;  /*!<Output Compare 4 Clear Enable */

} PACKSTRUCT TIM_CCMR2;
/*******************  Bit definition for TIM_CCER register  *******************/
typedef struct {
__IO ONEBIT  CC1E        :1;  /*!<Capture/Compare 1 output enable */
__IO ONEBIT  CC1P        :1;  /*!<Capture/Compare 1 output Polarity */
__IO ONEBIT  CC1NE       :1;  /*!<Capture/Compare 1 Complementary output enable */
__IO ONEBIT  CC1NP       :1;  /*!<Capture/Compare 1 Complementary output Polarity */
__IO ONEBIT  CC2E        :1;  /*!<Capture/Compare 2 output enable */
__IO ONEBIT  CC2P        :1;  /*!<Capture/Compare 2 output Polarity */
__IO ONEBIT  CC2NE       :1;  /*!<Capture/Compare 2 Complementary output enable */
__IO ONEBIT  CC2NP       :1;  /*!<Capture/Compare 2 Complementary output Polarity */
__IO ONEBIT  CC3E        :1;  /*!<Capture/Compare 3 output enable */
__IO ONEBIT  CC3P        :1;  /*!<Capture/Compare 3 output Polarity */
__IO ONEBIT  CC3NE       :1;  /*!<Capture/Compare 3 Complementary output enable */
__IO ONEBIT  CC3NP       :1;  /*!<Capture/Compare 3 Complementary output Polarity */
__IO ONEBIT  CC4E        :1;  /*!<Capture/Compare 4 output enable */
__IO ONEBIT  CC4P        :1;  /*!<Capture/Compare 4 output Polarity */
__IO ONEBIT  CC4NP       :1;  /*!<Capture/Compare 4 Complementary output Polarity */
} PACKSTRUCT TIM_CCER;

/*******************  Bit definition for TIM_BDTR register  *******************/
typedef struct {
__IO uint16_t  DTG             :8;  /*!<DTG[0:7] bits (Dead-Time Generator set-up) */

__IO uint16_t  LOCK            :2;  /*!<LOCK[1:0] bits (Lock Configuration) */
__IO ONEBIT  OSSI            :1;  /*!<Off-State Selection for Idle mode */
__IO ONEBIT  OSSR            :1;  /*!<Off-State Selection for Run mode */
__IO ONEBIT  BKE             :1;  /*!<Break enable */
__IO ONEBIT  BKP             :1;  /*!<Break Polarity */
__IO ONEBIT  AOE             :1;  /*!<Automatic Output enable */
__IO ONEBIT  MOE             :1;  /*!<Main Output enable */
} PACKSTRUCT TIM_BDTR;

typedef union {
  __IO uint16_t VAL;
  __IO TIM_CR1  BIT;
} TIM_CR1_REG;
typedef union {
  __IO uint16_t VAL;
  __IO TIM_CR2  BIT;
} TIM_CR2_REG;

typedef union {
  __IO uint16_t VAL;
  __IO TIM_SR   BIT;
} TIM_SR_REG;

typedef union {
  __IO uint16_t VAL;
  __IO TIM_EGR  BIT;
} TIM_EGR_REG;

typedef union {
  __IO uint16_t VAL;
  __IO TIM_DIER BIT;
} TIM_DIER_REG;

typedef union {
  __IO uint16_t VAL;
  __IO TIM_CCER BIT;
} TIM_CCER_REG;

typedef union {
  __IO uint16_t   VAL;
  __IO TIM_CCMR1  BIT;
} TIM_CCMR1_REG;

typedef union {
  __IO uint16_t   VAL;
  __IO TIM_CCMR2  BIT;
} TIM_CCMR2_REG;

typedef union {
  __IO uint16_t   VAL;
  __IO TIM_BDTR  BIT;
} TIM_BDTR_REG;


typedef struct {
  __IO TIM_CR1_REG CR1;          /*!< TIM control register 1,                      Address offset: 0x00 */
  uint16_t      RESERVED0;       /*!< Reserved,                                                    0x02 */
  __IO TIM_CR2_REG CR2;          /*!< TIM control register 2,                      Address offset: 0x04 */
  uint16_t      RESERVED1;       /*!< Reserved,                                                    0x06 */
  __IO uint16_t SMCR;            /*!< TIM slave Mode Control register,             Address offset: 0x08 */
  uint16_t      RESERVED2;       /*!< Reserved,                                                    0x0A */
  __IO TIM_DIER_REG DIER;        /*!< TIM DMA/interrupt enable register,           Address offset: 0x0C */
  uint16_t      RESERVED3;       /*!< Reserved,                                                    0x0E */
  __IO TIM_SR_REG  SR;           /*!< TIM status register,                         Address offset: 0x10 */
  uint16_t      RESERVED4;       /*!< Reserved,                                                    0x12 */
  __IO TIM_EGR_REG EGR;          /*!< TIM event generation register,               Address offset: 0x14 */
  uint16_t      RESERVED5;       /*!< Reserved,                                                    0x16 */
  __IO TIM_CCMR1_REG CCMR1;      /*!< TIM  capture/compare mode register 1,        Address offset: 0x18 */
  uint16_t      RESERVED6;       /*!< Reserved,                                                    0x1A */
  __IO TIM_CCMR2_REG CCMR2;      /*!< TIM  capture/compare mode register 2,        Address offset: 0x1C */
  uint16_t      RESERVED7;       /*!< Reserved,                                                    0x1E */
  __IO TIM_CCER_REG CCER;        /*!< TIM capture/compare enable register,         Address offset: 0x20 */
  uint16_t      RESERVED8;       /*!< Reserved,                                                    0x22 */
  __IO uint32_t CNT;             /*!< TIM counter register,                        Address offset: 0x24 */
  __IO uint16_t PSC;             /*!< TIM prescaler register,                      Address offset: 0x28 */
  uint16_t      RESERVED10;      /*!< Reserved,                                                    0x2A */
  __IO uint32_t ARR;             /*!< TIM auto-reload register,                    Address offset: 0x2C */
  __IO uint16_t RCR;             /*!< TIM  repetition counter register,            Address offset: 0x30 */
  uint16_t      RESERVED12;      /*!< Reserved,                                                    0x32 */
  __IO uint32_t CCR1;            /*!< TIM capture/compare register 1,              Address offset: 0x34 */
  __IO uint32_t CCR2;            /*!< TIM capture/compare register 2,              Address offset: 0x38 */
  __IO uint32_t CCR3;            /*!< TIM capture/compare register 3,              Address offset: 0x3C */
  __IO uint32_t CCR4;            /*!< TIM capture/compare register 4,              Address offset: 0x40 */
  __IO TIM_BDTR_REG BDTR;        /*!< TIM break and dead-time register,            Address offset: 0x44 */
  uint16_t      RESERVED17;      /*!< Reserved,                                                    0x26 */
  __IO uint16_t DCR;             /*!< TIM DMA control register,                    Address offset: 0x48 */
  uint16_t      RESERVED18;      /*!< Reserved,                                                    0x4A */
  __IO uint16_t DMAR;            /*!< TIM DMA address for full transfer register,  Address offset: 0x4C */
  uint16_t      RESERVED19;      /*!< Reserved,                                                    0x4E */
  __IO uint16_t OR;              /*!< TIM option register,                         Address offset: 0x50 */
  uint16_t      RESERVED20;      /*!< Reserved,                                                    0x52 */
} TIM_Type;

#endif // TIMER_REG_H
