#ifndef USART1CLASS_H
#define USART1CLASS_H

#include "baselayer.h"
#include "fifo.h"
/** @file
  * @brief Inicializace a obsluha USART1.
  
  * @class Usart1Class
  * @brief Ve stacku dole - Bottom, pod tím jsou už jen dráty.
  * 
  * Ukazuje použití jak dědičnosti - dědí vlastnosti třídy BaseLayer, tak polymorfizmu,
  * protože BaseLayer používá virtuální veřejné metody pro začlenění do stacku.
  * Kromě toho je sem zapouzdřena (kompozice) i třída Fifo, což je šablona.
  * Takže deklarace na několika řádcích obsahuje snad všechny vymoženosti,
  * které C++ nabízí a přitom bude generovat smysluplný a celkem použitelný kód.
  
  * Není to psáno obecně jako třeba periferní knihovny od ST, ale to je úmysl,
  * kód by měl být co nejmenší a přesto opakovatelně použitelný. Prakticky je potřeba
  * opravdu nastavit jen přenosovou rychlost, málokdy je potřeba měnit formát 8N1
  * nebo zapínat hardware flow control. A USART na tomto procesoru je tak složitý,
  * že prakticky všechny jeho speciální módy činnosti knihovna ST stejně neumí postihnout.
  */
/** 
  * @brief Universal Synchronous Asynchronous Receiver Transmitter
  */
#include "common.h"

/******************  Bit definition for USART_CR1 register  *******************/
typedef struct {
__IO ONEBIT  UE          :1;    /*!< USART Enable */
__IO ONEBIT  UESM        :1;    /*!< USART Enable in STOP Mode */
__IO ONEBIT  RE          :1;    /*!< Receiver Enable */
__IO ONEBIT  TE          :1;    /*!< Transmitter Enable */
__IO ONEBIT  IDLEIE      :1;    /*!< IDLE Interrupt Enable */
__IO ONEBIT  RXNEIE      :1;    /*!< RXNE Interrupt Enable */
__IO ONEBIT  TCIE        :1;    /*!< Transmission Complete Interrupt Enable */
__IO ONEBIT  TXEIE       :1;    /*!< TXE Interrupt Enable */

__IO ONEBIT  PEIE        :1;    /*!< PE Interrupt Enable */
__IO ONEBIT  PS          :1;    /*!< Parity Selection */
__IO ONEBIT  PCE         :1;    /*!< Parity Control Enable */
__IO ONEBIT  WAKE        :1;    /*!< Receiver Wakeup method */
__IO ONEBIT  M           :1;    /*!< Word length */
__IO ONEBIT  MME         :1;    /*!< Mute Mode Enable */
__IO ONEBIT  CMIE        :1;    /*!< Character match interrupt enable */
__IO ONEBIT  OVER8       :1;    /*!< Oversampling by 8-bit or 16-bit mode */

__IO uint32_t  DEDT      :4;    /*!< DEDT[4:0] bits (Driver Enable Deassertion Time) */
__IO uint32_t  DEAT      :4;    /*!< DEAT[4:0] bits (Driver Enable Assertion Time) */

__IO ONEBIT  RTOIE       :1;    /*!< Receive Time Out interrupt enable */
__IO ONEBIT  EOBIE       :1;    /*!< End of Block interrupt enable */
     uint32_t  UNUSED1   :6;
} USART_CR1;
/******************  Bit definition for USART_CR2 register  *******************/
typedef struct {
     uint32_t  UNUSED1     :4;
__IO ONEBIT  ADDM7       :1;    /*!< 7-bit or 4-bit Address Detection */
__IO ONEBIT  LBDL        :1;    /*!< LIN Break Detection Length */
__IO ONEBIT  LBDIE       :1;    /*!< LIN Break Detection Interrupt Enable */
     uint32_t  UNUSED2   :1;
     
__IO ONEBIT  LBCL        :1;    /*!< Last Bit Clock pulse */
__IO ONEBIT  CPHA        :1;    /*!< Clock Phase */
__IO ONEBIT  CPOL        :1;    /*!< Clock Polarity */
__IO ONEBIT  CLKEN       :1;    /*!< Clock Enable */
__IO uint32_t  STOP        :2;    /*!< STOP[1:0] bits (STOP bits) */
__IO ONEBIT  LINEN       :1;    /*!< LIN mode enable */
__IO ONEBIT  SWAP        :1;    /*!< SWAP TX/RX pins */

__IO ONEBIT  RXINV       :1;    /*!< RX pin active level inversion */
__IO ONEBIT  TXINV       :1;    /*!< TX pin active level inversion */
__IO ONEBIT  DATAINV     :1;    /*!< Binary data inversion */
__IO ONEBIT  MSBFIRST    :1;    /*!< Most Significant Bit First */
__IO ONEBIT  ABREN       :1;    /*!< Auto Baud-Rate Enable*/
__IO uint32_t  ABRMODE     :2;    /*!< ABRMOD[1:0] bits (Auto Baud-Rate Mode) */
__IO ONEBIT  RTOEN       :1;    /*!< Receiver Time-Out enable */

__IO uint32_t  ADD         :8;    /*!< Address of the USART node */
} USART_CR2;

/******************  Bit definition for USART_CR3 register  *******************/
typedef struct {
__IO ONEBIT  EIE         :1;    /*!< Error Interrupt Enable */
__IO ONEBIT  IREN        :1;    /*!< IrDA mode Enable */
__IO ONEBIT  IRLP        :1;    /*!< IrDA Low-Power */
__IO ONEBIT  HDSEL       :1;    /*!< Half-Duplex Selection */
__IO ONEBIT  NACK        :1;    /*!< SmartCard NACK enable */
__IO ONEBIT  SCEN        :1;    /*!< SmartCard mode enable */
__IO ONEBIT  DMAR        :1;    /*!< DMA Enable Receiver */
__IO ONEBIT  DMAT        :1;    /*!< DMA Enable Transmitter */

__IO ONEBIT  RTSE        :1;    /*!< RTS Enable */
__IO ONEBIT  CTSE        :1;    /*!< CTS Enable */
__IO ONEBIT  CTSIE       :1;    /*!< CTS Interrupt Enable */
__IO ONEBIT  ONE1BIT     :1;    /*!< One sample bit method enable */
__IO ONEBIT  OVRDIS      :1;    /*!< Overrun Disable */
__IO ONEBIT  DDRE        :1;    /*!< DMA Disable on Reception Error */
__IO ONEBIT  DEM         :1;    /*!< Driver Enable Mode */
__IO ONEBIT  DEP         :1;    /*!< Driver Enable Polarity Selection */

     uint32_t  UNUSED1     :1;
__IO uint32_t  SCARCNT     :3;    /*!< SCARCNT[2:0] bits (SmartCard Auto-Retry Count) */
__IO uint32_t  WUS         :2;    /*!< WUS[1:0] bits (Wake UP Interrupt Flag Selection) */
__IO ONEBIT  WUFIE       :1;    /*!< Wake Up Interrupt Enable */
     uint32_t  UNUSED2     :9;
} USART_CR3;

/******************  Bit definition for USART_BRR register  *******************/
#define  USART_BRR_DIV_FRACTION              ((uint16_t)0x000F)                /*!< Fraction of USARTDIV */
#define  USART_BRR_DIV_MANTISSA              ((uint16_t)0xFFF0)                /*!< Mantissa of USARTDIV */

/******************  Bit definition for USART_GTPR register  ******************/
#define  USART_GTPR_PSC                      ((uint16_t)0x00FF)                /*!< PSC[7:0] bits (Prescaler value) */
#define  USART_GTPR_GT                       ((uint16_t)0xFF00)                /*!< GT[7:0] bits (Guard time value) */


/*******************  Bit definition for USART_RTOR register  *****************/
#define  USART_RTOR_RTO                      ((uint32_t)0x00FFFFFF)            /*!< Receiver Time Out Value */
#define  USART_RTOR_BLEN                     ((uint32_t)0xFF000000)            /*!< Block Length */

/*******************  Bit definition for USART_RQR register  ******************/
#define  USART_RQR_ABRRQ                    ((uint16_t)0x0001)                /*!< Auto-Baud Rate Request */
#define  USART_RQR_SBKRQ                    ((uint16_t)0x0002)                /*!< Send Break Request */
#define  USART_RQR_MMRQ                     ((uint16_t)0x0004)                /*!< Mute Mode Request */
#define  USART_RQR_RXFRQ                    ((uint16_t)0x0008)                /*!< Receive Data flush Request */
#define  USART_RQR_TXFRQ                    ((uint16_t)0x0010)                /*!< Transmit data flush Request */

/*******************  Bit definition for USART_ISR register  ******************/
typedef struct {
__IO ONEBIT  PE           :1;    /*!< Parity Error */
__IO ONEBIT  FE           :1;    /*!< Framing Error */
__IO ONEBIT  NE           :1;    /*!< Noise detected Flag */
__IO ONEBIT  ORE          :1;    /*!< OverRun Error */
__IO ONEBIT  IDLE         :1;    /*!< IDLE line detected */
__IO ONEBIT  RXNE         :1;    /*!< Read Data Register Not Empty */
__IO ONEBIT  TC           :1;    /*!< Transmission Complete */
__IO ONEBIT  TXE          :1;    /*!< Transmit Data Register Empty */

__IO ONEBIT  LBD          :1;    /*!< LIN Break Detection Flag */
__IO ONEBIT  CTSIF        :1;    /*!< CTS interrupt flag */
__IO ONEBIT  CTS          :1;    /*!< CTS flag */
__IO ONEBIT  RTOF         :1;    /*!< Receiver Time Out */
__IO ONEBIT  EOBF         :1;    /*!< End Of Block Flag */
     uint32_t  UNUSED1    :1;
__IO ONEBIT  ABRE         :1;    /*!< Auto-Baud Rate Error */
__IO ONEBIT  ABRF         :1;    /*!< Auto-Baud Rate Flag */

__IO ONEBIT  BUSY         :1;    /*!< Busy Flag */
__IO ONEBIT  CMF          :1;    /*!< Character Match Flag */
__IO ONEBIT  SBKF         :1;    /*!< Send Break Flag */
__IO ONEBIT  RWU          :1;    /*!< Receive Wake Up from mute mode Flag */
__IO ONEBIT  WUF          :1;    /*!< Wake Up from stop mode Flag */
__IO ONEBIT  TEACK        :1;    /*!< Transmit Enable Acknowledge Flag */
__IO ONEBIT  REACK        :1;    /*!< Receive Enable Acknowledge Flag */
     uint32_t  UNUSED2      :9;
} USART_ISR;
/*******************  Bit definition for USART_ICR register  ******************/
typedef struct {
__IO ONEBIT  PECF         :1;    /*!< Parity Error Clear Flag */
__IO ONEBIT  FECF         :1;    /*!< Framing Error Clear Flag */
__IO ONEBIT  NCF          :1;    /*!< Noise detected Clear Flag */
__IO ONEBIT  ORECF        :1;    /*!< OverRun Error Clear Flag */
__IO ONEBIT  IDLECF       :1;    /*!< IDLE line detected Clear Flag */
     uint32_t  UNUSED1    :1;
__IO ONEBIT  TCCF         :1;    /*!< Transmission Complete Clear Flag */
     uint32_t  UNUSED2    :1;
     
__IO ONEBIT  LBDCF        :1;    /*!< LIN Break Detection Clear Flag */
__IO ONEBIT  CTSCF        :1;    /*!< CTS Interrupt Clear Flag */
     uint32_t  UNUSED3    :1;
__IO ONEBIT  RTOCF        :1;    /*!< Receiver Time Out Clear Flag */
__IO ONEBIT  EOBCF        :1;    /*!< End Of Block Clear Flag */
     uint32_t  UNUSED4      :4;
__IO ONEBIT  CMCF         :1;    /*!< Character Match Clear Flag */
     uint32_t  UNUSED5      :2;
__IO ONEBIT  WUCF         :1;    /*!< Wake Up from stop mode Clear Flag */
     uint32_t  UNUSED6      :11;
} USART_ICR;

typedef union {
  __IO uint32_t         VAL;
  __IO USART_CR1        BIT;
} USART_CR1_REG;
typedef union {
  __IO uint32_t         VAL;
  __IO USART_CR2        BIT;
} USART_CR2_REG;
typedef union {
  __IO uint32_t         VAL;
  __IO USART_CR3        BIT;
} USART_CR3_REG;
typedef union {
  __IO uint32_t         VAL;
  __IO USART_ISR        BIT;
} USART_ISR_REG;
typedef union {
  __IO uint32_t         VAL;
  __IO USART_ICR        BIT;
} USART_ICR_REG;


typedef struct {
  __IO USART_CR1_REG CR1;    /*!< USART Control register 1,                 Address offset: 0x00 */ 
  __IO USART_CR2_REG CR2;    /*!< USART Control register 2,                 Address offset: 0x04 */ 
  __IO USART_CR3_REG CR3;    /*!< USART Control register 3,                 Address offset: 0x08 */
  __IO uint16_t      BRR;    /*!< USART Baud rate register,                 Address offset: 0x0C */
  uint16_t  RESERVED1;       /*!< Reserved, 0x0E                                                 */  
  __IO uint16_t      GTPR;   /*!< USART Guard time and prescaler register,  Address offset: 0x10 */
  uint16_t  RESERVED2;       /*!< Reserved, 0x12                                                 */
  __IO uint32_t      RTOR;   /*!< USART Receiver Time Out register,         Address offset: 0x14 */  
  __IO uint16_t      RQR;    /*!< USART Request register,                   Address offset: 0x18 */
  uint16_t  RESERVED3;       /*!< Reserved, 0x1A                                                 */
  __IO USART_ISR_REG ISR;    /*!< USART Interrupt and status register,      Address offset: 0x1C */
  __IO USART_ICR_REG ICR;    /*!< USART Interrupt flag Clear register,      Address offset: 0x20 */
  __IO uint16_t      RDR;    /*!< USART Receive Data register,              Address offset: 0x24 */
  uint16_t  RESERVED4;       /*!< Reserved, 0x26                                                 */
  __IO uint16_t      TDR;    /*!< USART Transmit Data register,             Address offset: 0x28 */
  uint16_t  RESERVED5;       /*!< Reserved, 0x2A                                                 */
} USART_Type;
//! [Usart example]
class Usart1Class : public BaseLayer {
  public:
    /// Konstruktor @param baud Přenosová rychlost.
    Usart1Class         (uint32_t baud);
    /// Přetížení - data (přes FIFO) přímo na drát
    /// @param data ukazatel na data
    /// @param len  délka dat v bytech
    /// @return počet skutečně přenesených bytů (může se od len dost lišit)
    uint32_t    Down    (const char* data, uint32_t len);
    /// Takhle by se daly přidávat další nastavení 
    void        SetHalfDuplex (bool on) const;
    static void irq (void);
  public:   // necháme veřejné
    static Fifo<char>   tx; //!< Zajišťuje, že metoda Down neblokuje program
    static Usart1Class* inst;
};
//! [Usart example]

#endif // USART1CLASS_H
