#include "adcirq.h"
#include "gpio.h"
#include "rcc.h"
#include "timreg.h"
#include "adcreg.h"

// Statické členy třídy.
AdcIrq*       AdcIrq::instance  = 0;
// Pomocné odkazy přímo na HW.
static TIM_Type & TIM3 = *(TIM_Type*) 0x40000400;
static ADC_Type & ADC1 = *(ADC_Type*) 0x40012400;

// Několik pomocných nečlenských inline funkcí nad HW. //////////////////
static inline void EnableClock (void) {
  RCC->APB2ENR.BIT.ADCEN  = SET;
  RCC->APB1ENR.BIT.TIM3EN = SET;
}
static inline void Timer3Init (uint32_t us) {
  TIM3.PSC  = 47;
  TIM3.ARR  = us - 1;
  TIM3.RCR  = 0;
  // Preload, enable
  TIM3.CR1.BIT.ARPE = SET;
  // TRGO update for ADC
  TIM3.CR2.BIT.MMS  = 2;
}
static inline void AdcCalibrate (void) {
  // Ensure that ADEN = 0
  // Clear ADEN
  if (ADC1.CR.BIT.ADEN != RESET) {
    ADC1.CR.BIT.ADEN = RESET;
  }
  // Launch the calibration by setting ADCAL
  ADC1.CR.BIT.ADCAL = SET;
  // Wait until ADCAL=0
  while (ADC1.CR.BIT.ADCAL != RESET);
  //__NOP();
  //__NOP();   // This 2 NOPs are to ensure 2 ADC Cycles  before setting ADEN bit
}
static inline void AdcInit (uint32_t ch) {
  // PCLK / 2 - jitter
  ADC1.CFGR2.BIT.CKMODE = 1;
  // Select TRG TIM3
  register ADC_CFGR1_REG cfgr;
  cfgr.VAL = ADC1.CFGR1.VAL;
  cfgr.BIT.EXTEN  = 1;
  cfgr.BIT.EXTSEL = 3;
  cfgr.BIT.RES    = 0;          // 12.bit
  ADC1.CFGR1.VAL  = cfgr.VAL;
  // Select CHSELx
  ADC1.CHSELR = (1<<ch);
  // Select a sampling mode of 000
  ADC1.SMPR            = 5;     // 55.5 cycles
  ADC1.IER.BIT.EOCIE   = SET;
}
static inline void AdcStart (void) {
  ADC1.CR. BIT.ADEN    = SET;
  TIM3.CR1.BIT.CEN     = SET;
  ADC1.CR. BIT.ADSTART = SET;
}
/////////////////////////////////////////////////////////////////////////
// Vlastní konstruktor.
/////////////////////////////////////////////////////////////////////////
AdcIrq::AdcIrq (uint32_t us, uint32_t ch) : ProcessData () {
  if (instance) return;         // Chyba - musí být právě jedna instance
  instance = this;              // a to tato
  EnableClock ();
  AdcCalibrate();
  GpioClass in (GpioPortA, ch, GPIO_Mode_AN);
  Timer3Init  (us);
  NVIC_EnableIRQ (ADC1_COMP_IRQn);
  AdcInit     (ch);
  AdcStart    ();
}
/////////////////////////////////////////////////////////////////////////
// A obsluha přerušení.
/////////////////////////////////////////////////////////////////////////
void AdcIrq::Irq (void) {
  uint16_t data;
  data = (uint16_t) ADC1.DR;
  instance->Compute (data);
}
// Finta - přejmenování interrupt handleru, aby se jeho jméno nemuselo měnit ve startup.c.
extern "C" void ADC1_COMP_IRQHandler (void) __attribute__((alias("_ZN6AdcIrq3IrqEv")));
/***********************************************************************/
