#include "gpio.h"
#include "usart1.h"
#include "rcc.h"

USART_Type * const USART1 = (USART_Type*) 0x40013800;

Usart1Class* Usart1Class::inst = 0;
Fifo<char>   Usart1Class::tx;
//![USART1_IRQHandler example]
// A jeho obsluha přerušení.
void Usart1Class::irq (void) {
  volatile register USART_ISR_REG status;
  char rdata, tdata;
  
  status.VAL = USART1->ISR.VAL;                 // načti status přerušení
  if (status.BIT.TXE) {                         // od vysílače
    if (tx.Read (tdata))                        // pokud máme data
      USART1->TDR = (uint32_t) tdata & 0xFF;      // zapíšeme do výstupu
    else                                        // pokud ne
      USART1->CR1.BIT.TXEIE = RESET;              // je nutné zakázat přerušení od vysílače
  }
  if (status.BIT.RXNE) {                        // od přijímače
    rdata = (USART1->RDR) & 0xFF;               // načteme data
    inst->Up (&rdata, 1);
  }
}
/// Voláno z čistého C - startup.c
extern "C" void USART1_IRQHandler (void)__attribute__((alias("_ZN11Usart1Class3irqEv")));
//![USART1_IRQHandler example]
extern "C" uint32_t SystemCoreClock;
// Class is Bottom
Usart1Class::Usart1Class (uint32_t baud) : BaseLayer() {
  if (inst) return;     // Chyba - jedina instance
  inst = this;
  // 1. Clock Enable
  RCC->APB2ENR.BIT.USART1EN = SET;
  // 2. GPIO Alternate Config
  GpioClass txp (GpioPortA,  9, GPIO_Mode_AF);
  GpioClass rxp (GpioPortA, 10, GPIO_Mode_AF);
  txp.setAF (1);
  rxp.setAF (1);
  // 4. NVIC
  NVIC_EnableIRQ (USART1_IRQn);
  uint32_t tmp = 0;
  // 5. USART registry 8.bit bez parity
  register USART_CR1_REG cr1;
  cr1.BIT.RE      = SET;
  cr1.BIT.TE      = SET;
  cr1.BIT.RXNEIE  = SET;
  USART1->CR1.VAL = cr1.VAL;
  USART1->CR2.VAL = 0;
  USART1->CR3.BIT.OVRDIS = SET;
  // Tuhle část už vezmeme přímo z knihovny, jen ty hodiny zjednodušíme na SystemCoreClock
  uint32_t apbclock = SystemCoreClock;
  uint32_t integerdivider, fractionaldivider;
  /* Determine the integer part */
  if (USART1->CR1.BIT.OVER8 != RESET) {
    /* Integer part computing in case Oversampling mode is 8 Samples */
    integerdivider = ((25 * apbclock) / (2 * (baud)));    
  } else  {
    /* Integer part computing in case Oversampling mode is 16 Samples */
    integerdivider = ((25 * apbclock) / (4 * (baud)));    
  }
  tmp = (integerdivider / 100) << 4;
  /* Determine the fractional part */
  fractionaldivider = integerdivider - (100 * (tmp >> 4));
  /* Implement the fractional part in the register */
  if (USART1->CR1.BIT.OVER8 != RESET) {
    tmp |= ((((fractionaldivider * 8) +  50) / 100)) & ((uint8_t)0x07);
  } else {
    tmp |= ((((fractionaldivider * 16) + 50) / 100)) & ((uint8_t)0x0F);
  } 
  /* Write to USART BRR */
  USART1->BRR = (uint16_t)tmp;
  USART1->CR1.BIT.UE = SET;        // nakonec povolit globálně
}
// Nahustí data do fifo, na nic nečeká
uint32_t Usart1Class::Down (const char* data, uint32_t len) {
  uint32_t res;
  for (res=0; res<len; res++) if (!tx.Write(data[res])) break;
  USART1->CR1.BIT.TXEIE = SET;      // po povolení přerušení okamžitě přeruší
  return res; // Je naspodu - Bottom, takže dál už není třeba nikam předávat
}
// Podobných funkcí pro nastavení je možné přidat více. Garbage collector je případně odstraní.
void Usart1Class::SetHalfDuplex (bool on) const {
  USART1->CR1.BIT.UE = RESET;       // zakázat, jinak nelze nastavovat
  if (on)
    USART1->CR3.BIT.HDSEL = SET;    // poloduplex on
  else
    USART1->CR3.BIT.HDSEL = RESET;  // poloduplex off
  USART1->CR1.BIT.UE = SET;         // nakonec povolit globálně
}
